package ca.recrown.islandsurvivalcraft.worldgen;

import java.util.Random;

import org.bukkit.Material;
import org.bukkit.generator.ChunkGenerator.ChunkData;
import org.bukkit.util.noise.PerlinOctaveGenerator;

import nl.rutgerkok.worldgeneratorapi.BaseTerrainGenerator;

public class IslandBaseTerrainGenerator implements BaseTerrainGenerator {
    private int seaLevel;
    private int minimumHeight;
    private int maximumHeight;

    private PerlinOctaveGenerator noiseGenerator;

    public IslandBaseTerrainGenerator(long seed, int seaLevel, int seaFloor, int maximumHeight) {
        Random random = new Random(seed);
        this.seaLevel = seaLevel;
        this.minimumHeight = seaFloor;
        this.maximumHeight = maximumHeight;

        this.noiseGenerator = new PerlinOctaveGenerator(random, 8);
        noiseGenerator.setScale(0.025D);
    }

    @Override
    public void setBlocksInChunk(GeneratingChunk chunk) {
        
        for (int x = 0; x < 16; x++)
            for (int z = 0; z < 16; z++) {
                int worldX = 16 * chunk.getChunkX() + x;
                int worldZ = 16 * chunk.getChunkZ() + z;

                int height = baseHeight(worldX, worldZ);
                fillChunkColumn(x, z, height, chunk.getBlocksForChunk());
            }
    }

    private void fillChunkColumn(int x, int z, int height, ChunkData chunkData) {
        if (height <= seaLevel) {
            chunkData.setRegion(x, height, z, x+1, seaLevel, z+1, Material.WATER);
            chunkData.setRegion(x, 0, z, x+1, height, z+1, Material.STONE);
        } else {
            chunkData.setRegion(x, 0, z, x+1, height, z+1, Material.STONE);
        }
    }

    private int baseHeight(int x, int z) {
        return (int) ((noiseGenerator.noise(x, z, 0.5D, 0.5D, true) + 1D) * maximumHeight/2D) + minimumHeight;
    }

    @Override
    public int getHeight(int x, int z, HeightType type) {
        int baseHeight = baseHeight(x, z);
        if (baseHeight <= seaLevel && type == HeightType.WORLD_SURFACE) {
            return seaLevel;
        }
        return baseHeight;
    }

}